--[[

Skillet: A tradeskill window replacement.
Copyright (c) 2007 Robert Clark <nogudnik@gmail.com>

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

]]--

local L = AceLibrary("AceLocale-2.2"):new("Skillet")

SKILLET_TRADE_SKILLS_DISPLAYED = 26;
SKILLET_QUEUE_ITEMS_DISPLAYED = 5;
SKILLET_TRADE_SKILL_HEIGHT = 16;
SKILLET_NUM_REAGENT_BUTTONS = 8;

-- Width of skill buttons when there is no scroll bar
local NORMAL_BUTTON_WIDTH = nil
-- Width of skill buttons when the scroll bar is visible
local SCROLLED_BUTTON_WIDTH = nil

local SkilletSkillTypeColor = { };
SkilletSkillTypeColor["optimal"]	= { r = 1.00, g = 0.50, b = 0.25 };
SkilletSkillTypeColor["medium"]		= { r = 1.00, g = 1.00, b = 0.00 };
SkilletSkillTypeColor["easy"]		= { r = 0.25, g = 0.75, b = 0.25 };
SkilletSkillTypeColor["trivial"]	= { r = 0.50, g = 0.50, b = 0.50 };
SkilletSkillTypeColor["header"]		= { r = 1.00, g = 0.82, b = 0 };

-- Events
local AceEvent = AceLibrary("AceEvent-2.0")

-- Stack of porevisouly selected recipes for use by the
-- "click on reagent, go to recipe" code
local previousRecipies = {}
local gearTexture

-- Stolen from the Waterfall Ace2 addon.
local ControlBackdrop  = {
	bgFile = "Interface\\Tooltips\\UI-Tooltip-Background",
	edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
	tile = true, tileSize = 16, edgeSize = 16,
	insets = { left = 3, right = 3, top = 3, bottom = 3 }
}
local FrameBackdrop = {
	bgFile = "Interface\\Tooltips\\UI-Tooltip-Background",
	edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
	tile = true, tileSize = 16, edgeSize = 16,
	insets = { left = 3, right = 3, top = 30, bottom = 3 }
}

function Skillet:CreateTradeSkillWindow(parent)

	-- The SkilletFrame is defined in the file main_frame.xml
	local frame = SkilletFrame
	if not frame then
		return frame
	end
    
    if TradeJunkieMain and TJ_OpenButtonTradeSkill then        
        self:AddButtonToTradeskillWindow(TJ_OpenButtonTradeSkill)
    end
    if AC_Craft and AC_UseButton and AC_ToggleButton then
        self:AddButtonToTradeskillWindow(AC_UseButton)        
    end
    
	frame:SetBackdrop(FrameBackdrop);
	frame:SetBackdropColor(0.1, 0.1, 0.1)

	-- A title bar stolen from the Ace2 Waterfall window.
	local r,g,b = 0, 0.7, 0; -- dark green
	local titlebar = frame:CreateTexture(nil,"BACKGROUND")
	local titlebar2 = frame:CreateTexture(nil,"BACKGROUND")

	titlebar:SetPoint("TOPLEFT",frame,"TOPLEFT",3,-4)
	titlebar:SetPoint("TOPRIGHT",frame,"TOPRIGHT",-3,-4)
	titlebar:SetHeight(13)

	titlebar2:SetPoint("TOPLEFT",titlebar,"BOTTOMLEFT",0,0)
	titlebar2:SetPoint("TOPRIGHT",titlebar,"BOTTOMRIGHT",0,0)
	titlebar2:SetHeight(13)

	titlebar:SetGradientAlpha("VERTICAL",r*0.6,g*0.6,b*0.6,1,r,g,b,1)
	titlebar:SetTexture(r,g,b,1)
	titlebar2:SetGradientAlpha("VERTICAL",r*0.9,g*0.9,b*0.9,1,r*0.6,g*0.6,b*0.6,1)
	titlebar2:SetTexture(r,g,b,1)

	local title = CreateFrame("Frame",nil,frame)
	title:SetPoint("TOPLEFT",titlebar,"TOPLEFT",0,0)
	title:SetPoint("BOTTOMRIGHT",titlebar2,"BOTTOMRIGHT",0,0)

	local titletext = title:CreateFontString("SkilletTitleText", "OVERLAY", "GameFontNormalLarge")
	titletext:SetPoint("TOPLEFT",title,"TOPLEFT",0,0)
	titletext:SetPoint("TOPRIGHT",title,"TOPRIGHT",0,0)
	titletext:SetHeight(26)
	titletext:SetShadowColor(0,0,0)
	titletext:SetShadowOffset(1,-1)
	titletext:SetTextColor(1,1,1)
	titletext:SetText(L["Skillet Trade Skills"]);

	local label = getglobal("SkilletFilterLabel");
	label:SetText(L["Filter"]);

	SkilletCreateAllButton:SetText(L["Create All"])
    SkilletQueueAllButton:SetText(L["Queue All"])
    SkilletCreateButton:SetText(L["Create"])
    SkilletQueueButton:SetText(L["Queue"])
    SkilletStartQueueButton:SetText(L["Start Queue"])
    SkilletEmptyQueueButton:SetText(L["Empty Queue"])
    SkilletEnchantButton:SetText(L["Enchant"])
    SkilletShowOptionsButton:SetText(L["Options"])
    SkilletRescanButton:SetText(L["Rescan"])
    SkilletRecipeNotesButton:SetText(L["Notes"])
    SkilletRecipeNotesButton:SetTextFontObject("GameFontNormalSmall")
    SkilletRecipeNotesFrameLabel:SetText(L["Notes"])

    SkilletHideUncraftableRecipesText:SetText(L["Hide uncraftable"])
    SkilletHideTrivialRecipesText:SetText(L["Hide trivial"])

	frame:SetScript("OnShow", function()
		PlaySound("igBackPackOpen");
	end)
	frame:SetScript("OnHide", function()
		PlaySound("igBackPackClose")
		-- so the rest of the Blizz API knows that it's closed
		if Skillet:IsCraft() then
			CloseCraft()
		else
			CloseTradeSkill()
		end
	end)

	-- Always want these visible.
	SkilletItemCountInputBox:SetText("1");
	SkilletCreateCountSlider:SetMinMaxValues(1, 20);
	SkilletCreateCountSlider:SetValue(1);
	SkilletCreateCountSlider:Show();
	SkilletCreateCountSliderThumb:Show();

	-- Progression status bar
	SkilletRankFrame:SetStatusBarColor(0.2, 0.2, 1.0, 1.0);
	SkilletRankFrameBackground:SetVertexColor(0.0, 0.0, 0.5, 0.2);

	-- The frame enclosing the scroll list needs a border and a background .....
	local backdrop = SkilletSkillListParent
	backdrop:SetBackdrop(ControlBackdrop)
	backdrop:SetBackdropBorderColor(0.6, 0.6, 0.6)
	backdrop:SetBackdropColor(0.05, 0.05, 0.05)
	backdrop:SetResizable(true)

	-- Frame enclosing the reagent list
	backdrop = SkilletReagentParent
	backdrop:SetBackdrop(ControlBackdrop)
	backdrop:SetBackdropBorderColor(0.6, 0.6, 0.6)
	backdrop:SetBackdropColor(0.05, 0.05, 0.05)
	backdrop:SetResizable(true)

	-- Frame enclosing the queue
	backdrop = SkilletQueueParent
	backdrop:SetBackdrop(ControlBackdrop)
	backdrop:SetBackdropBorderColor(0.6, 0.6, 0.6)
	backdrop:SetBackdropColor(0.05, 0.05, 0.05)
	backdrop:SetResizable(true)

	-- frame enclosing the pop out notes panel
	backdrop = SkilletRecipeNotesFrame
	backdrop:SetBackdrop(ControlBackdrop)
	backdrop:SetBackdropColor(0.1, 0.1, 0.1)
	backdrop:SetBackdropBorderColor(0.6, 0.6, 0.6)
	backdrop:SetResizable(true)
	backdrop:Hide() -- initially hidden

    gearTexture = SkilletReagentParent:CreateTexture(nil, "OVERLAY")
    gearTexture:SetTexture("Interface\\Icons\\Trade_Engineering")
    gearTexture:SetHeight(16)
    gearTexture:SetWidth(16)
    
	-- Set button sizes based on the size of the scroll window
	NORMAL_BUTTON_WIDTH = SkilletSkillList:GetWidth() - 10;
	SCROLLED_BUTTON_WIDTH = NORMAL_BUTTON_WIDTH - 20;

	-- Ace Window manager library, allows the window position (and size)
	-- to be automatically saved
	local windowManger = AceLibrary("Window-1.0")
	local tradeSkillLocation = {
		prefix = "tradeSkillLocation_"
	}
	windowManger:RegisterConfig(frame, parent.db.char, tradeSkillLocation)
	windowManger:RestorePosition(frame)  -- restores scale also
	windowManger:MakeDraggable(frame)

	return frame
end

-- Something has changed in the tradeskills, and the window needs to be updated
function Skillet:TradeSkillRank_Updated()
	local _, rank, maxRank = self:GetTradeSkillLine();

	if rank and maxRank then
		SkilletRankFrame:SetMinMaxValues(0, maxRank);
		SkilletRankFrame:SetValue(rank);
		SkilletRankFrameSkillRank:SetText(rank.."/"..maxRank);
	end

end

-- Someone dragged the slider or set the value programatically.
function Skillet:UpdateNumItemsSlider(item_count, clicked)
	local value = floor(item_count + 0.5);

	self.numItemsToCraft = value

	if SkilletCreateCountSlider:IsVisible() then
		SkilletItemCountInputBox:SetText("" .. value)
		if not clicked then
			SkilletCreateCountSlider:SetValue(value)
		end
	end
end

-- Called when the list of skills is scrolled
function Skillet:SkillList_OnScroll()
	Skillet:UpdateTradeSkillsWindow()
end

-- Called when the list of queued items is scrolled
function Skillet:QueueList_OnScroll()
	Skillet:UpdateQueueWindow()
end

-- Figures out whether or not the section a recipe
-- is in has been hidden (collapsed or filtered).
-- Headers are, by definition never hidden
local function is_hidden_skill(parent, skill_index)

	-- look up the info in stitch to avoid spamming the server with
	-- GetTradeSkillInfo() calls. It does not seem to like that
	local s = Skillet.stitch:GetItemDataByIndex(parent.currentTrade, skill_index);

	if not s then
		-- it's a header, headers are not skills and can never be hidden
		return false
	end

	-- it's a recipe, is it filtered out?
	if parent.filtertext and parent.filtertext ~= "" then
		-- plain text search only.
		local name = string.lower(s.name);
		local filter = string.lower(parent.filtertext);
		if string.find(name, filter, 1, true) == nil then
			-- no match against the filter .. buh-bye
			return true
		end
	end

	-- it's a recipe, work backwards to find the section it's
	-- in and see if that has been collapsed.
	for i=skill_index-1, 0, -1 do
		if Skillet.stitch:GetItemDataByIndex(parent.currentTrade, i) == nil then
			-- found the header
			local skillName, _ = Skillet:GetTradeSkillInfo(i);
			if (parent.headerCollapsedState and parent.headerCollapsedState[skillName]) then
				return true;
			end
			break;
		end
	end

	-- are we hiding anything that can't be created with the mats on this character?
	if s.numcraftablewbank == 0 and parent:GetTradeskillOption(parent.currentTrade, "hideuncraftable") then
		return true
	end

	-- are we hiding anything that is trivial (has no chance of giving a skill point)
	if s.difficulty == "trivial" and parent:GetTradeskillOption(parent.currentTrade, "hidetrivial") then
		return true
	end

	return false

end

local function Skillet_redo_the_update(self)
    AceEvent:CancelScheduledEvent("Skillet_redo_the_update")
    self:UpdateTradeSkillsWindow()
end

-- Updates the trade skill window whenever anything has changed,
-- number of skills, skill type, skill level, etc
function Skillet:UpdateTradeSkillsWindow()
    if not self.currentTrade or self.currentTrade == "UNKNOWN" then
    	-- nothing to see, nothing to update
    	self:SetSelectedSkill(nil)
        return
    end

    -- hide UI components that cannot be used for crafts and show that
    -- that are only applicable to trade skills, as needed
    if self:IsCraft() then
        SkilletQueueAllButton:Hide()
        SkilletQueueButton:Hide()
        SkilletCreateAllButton:Hide()
        SkilletCreateButton:Hide()
        SkilletCreateCountSlider:Hide()
        SkilletCreateCountSliderThumb:Hide()
        SkilletItemCountInputBox:Hide()
        SkilletQueueParent:Hide()
        SkilletStartQueueButton:Hide()
        SkilletEmptyQueueButton:Hide()

        SkilletEnchantButton:Show();
    else
        SkilletQueueAllButton:Show()
        SkilletQueueButton:Show()
        SkilletCreateAllButton:Show()
        SkilletCreateButton:Show()
        SkilletCreateCountSlider:Show()
        SkilletCreateCountSliderThumb:Show()
        SkilletItemCountInputBox:Show()
        SkilletQueueParent:Show()
        SkilletStartQueueButton:Show()
        SkilletEmptyQueueButton:Show()

        SkilletEnchantButton:Hide();
    end

    -- Set the state of any craft specific options
    SkilletHideTrivialRecipes:SetChecked(self:GetTradeskillOption(self.currentTrade, "hidetrivial"))
    SkilletHideUncraftableRecipes:SetChecked(self:GetTradeskillOption(self.currentTrade, "hideuncraftable"))

    self:UpdateQueueWindow()

    local _, rank, maxRank = self:GetTradeSkillLine();

    -- Window Title
    local title = getglobal("SkilletTitleText");
    if title then
    	title:SetText(L["Skillet Trade Skills"] .. ": " .. self.currentTrade)
    end

    local numTradeSkills = self:GetNumTradeSkills()

    -- Progression status bar
	SkilletRankFrame:SetMinMaxValues(0, maxRank);
	SkilletRankFrame:SetValue(rank);
	SkilletRankFrameSkillRank:SetText(rank.."/"..maxRank);

	-- Update the scroll frame
	FauxScrollFrame_Update(SkilletSkillList,				-- frame
	                       numTradeSkills,                  -- num items
	                       SKILLET_TRADE_SKILLS_DISPLAYED,  -- num to display
	                       SKILLET_TRADE_SKILL_HEIGHT)      -- value step (item height)

	-- Where in the list of skill to start counting.
	local skillOffset = FauxScrollFrame_GetOffset(SkilletSkillList);

	-- Remove any selected highlight, it will be added back as needed
	SkilletHighlightFrame:Hide();
    
    local nilFound = false

	-- Iterate through all the buttons that make up the scroll window
	-- and fill then in with data or hide them, as necessary
	for i=1, SKILLET_TRADE_SKILLS_DISPLAYED, 1 do
		local skillIndex = i + skillOffset;
		local button = getglobal("SkilletScrollButton"..i);

		-- set the width based on whether or not the scroll bar is displayed
		if ( SkilletSkillList:IsShown() ) then
			button:SetWidth(288);
		else
			button:SetWidth(308);
		end

		-- skip over any hidden skills
		while (skillIndex <= numTradeSkills) and is_hidden_skill(self, skillIndex) do
		 	skillOffset = skillOffset + 1;
		 	skillIndex = i + skillOffset;
		end

		if ( skillIndex <= numTradeSkills ) then
        
            local skillName, skillType = self:GetTradeSkillInfo(skillIndex);
            if not skillName then
                local s = self.stitch:GetItemDataByIndex(self.currentTrade, skillIndex)
                if s == nil then
                    skillType = "header"
                    skillName = ""
                    nilFound = true
                else
                    skillName = s.name
                end                
            end
            
            local color = SkilletSkillTypeColor[skillType];
            if (color) then
                button:SetTextColor(color.r, color.g, color.b);
            end

			if skillType == "header" then
				local collapsed = false;
				if self.headerCollapsedState and self.headerCollapsedState[skillName] then
					collapsed = self.headerCollapsedState[skillName]
				end

				if collapsed then
					-- Collapsed
					button:SetNormalTexture("Interface\\Buttons\\UI-PlusButton-Up");
				else
					-- expanded
					button:SetNormalTexture("Interface\\Buttons\\UI-MinusButton-Up");
				end

				button:SetText(skillName);
				button:SetID(-1);
				button:UnlockHighlight(); -- headers never get highlighted

				button:Show();

       		else
       			local color = SkilletSkillTypeColor[skillType];
				if (color) then
					button:SetTextColor(color.r, color.g, color.b);
				end

       			button:SetNormalTexture("");
				getglobal("SkilletScrollButton"..i.."Highlight"):SetTexture("");

       			local s = self.stitch:GetItemDataByIndex(self.currentTrade, skillIndex);

                local text = " ";
                if s then
                    -- if the item has a minimum level requirement, then print that here
                    if self.db.profile.display_required_level then
                        local level = self:GetLevelRequiredToUse(s.link)
                        if level and level > 1 then
                            text = text .. "[" .. level .. "] "
                        end
                    end
                    
                    text = text .. s.name
                    if s.numcraftable > 0 or s.numcraftablewbank > 0 or (s.numcraftablewalts and s.numcraftablewalts > 0) then
                        text = text .. " [" .. s.numcraftable .. "/" .. s.numcraftablewbank
                        if s.numcraftablewalts then
                            -- only show this if there is a mod installed that
                            -- allows stitch to collect the information.
                            text = text .. "/" .. s.numcraftablewalts
                        end
                        text = text .. "]"
                    end
                    button:SetID(skillIndex)
                else
                    nilFound = true
                    -- not cached yet
                end

				button:SetText(text);

				if ( self.selectedSkill and self.selectedSkill == skillIndex ) then
					-- user has this skill selected
					SkilletHighlightFrame:SetPoint("TOPLEFT", "SkilletScrollButton"..i, "TOPLEFT", 0, 0);
					SkilletHighlightFrame:SetWidth(button:GetWidth()-5);
					SkilletHighlightFrame:Show();
                    if color then
                        SkilletHighlight:SetTexture(color.r, color.g, color.b, 0.4)
                    else
                        SkilletHighlight:SetTexture(0.7, 0.7, 0.7, 0.4)
                    end               

					-- And update the details for this skill, just in case something
					-- has changed (mats consumed, etc)
					self:UpdateDetailsWindow(self.selectedSkill);

					button:LockHighlight();
				else
					-- not selected
					button:SetBackdropColor(0.8, 0.2, 0.2);
					button:UnlockHighlight();
				end

				button:Show();

			end
	    else
	    	-- We have no data for you Mister Button .....
	    	button:Hide();
	    	button:UnlockHighlight();
		end
    end

    if nilFound then        
        if not AceEvent:IsEventScheduled("Skillet_redo_the_update") then
            AceEvent:ScheduleEvent("Skillet_redo_the_update", 0.25, self)
        end
    end

end

-- Display an action packed tooltip when we are over
-- a recipe in the list of skills
--
-- id is the index of the skill in the currently selected trade.
function Skillet:DisplayTradeskillTooltip(id)

	if id < 0 then
		-- it's header or not cached yet.
		return
	end
    
    if not self.db.profile.show_detailed_recipe_tooltip then
        -- user does not want the tooltip displayed, it can get a bit big after all
        return
    end

	SkilletTradeskillTooltip:SetOwner(this, "ANCHOR_BOTTOMRIGHT",-300);
	SkilletTradeskillTooltip:SetBackdropColor(0,0,0,1);
	SkilletTradeskillTooltip:ClearLines();
    SkilletTradeskillTooltip:SetClampedToScreen(true)

	SkilletTradeskillTooltip:SetWidth(SkilletTradeskillTooltip:GetWidth() * 2);

	local s = self.stitch:GetItemDataByIndex(self.currentTrade, id)
    if not s then
        -- this can happen when the recipe is not yet cached
        return
    end   

	-- Name of the recipe

	local color = SkilletSkillTypeColor[s.difficulty];
    if (color) then
		SkilletTradeskillTooltip:AddLine(s.name, color.r, color.g, color.b, 0);
	else
		SkilletTradeskillTooltip:AddLine(s.name, NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b, 0);
	end

	-- how many can be created with the reagents in the inventory
	if s.numcraftable > 0 then
		local text = "\n" .. s.numcraftable .. " " .. L["can be created from reagents in your inventory"];
		SkilletTradeskillTooltip:AddLine(text, 1, 1, 1, 0); -- (text, r, g, b, wrap)
	end
	-- how many can be created with the reagent in your inv + bank
	if s.numcraftablewbank > 0 and s.numcraftablewbank ~= s.numcraftable then
		local text = s.numcraftablewbank .. " " .. L["can be created from reagents in your inventory and bank"];
		if s.numcraftable == 0 then
			text = "\n" .. text;
		end
		SkilletTradeskillTooltip:AddLine(text, 1, 1, 1, 0);	-- (text, r, g, b, wrap)
	end
	-- how many can be crafted with reagents on *all* alts, including this one.
	if s.numcraftablewalts and s.numcraftablewalts > 0 and s.numcraftablewalts ~= s.numcraftable then
		local text = s.numcraftablewalts .. " " .. L["can be created from reagents on all characters"];
		if s.numcraftable and s.numcraftablewbank == 0 then
			text = "\n" .. text;
		end
		SkilletTradeskillTooltip:AddLine(text, 1, 1, 1, 0);	-- (text, r, g, b, wrap)
	end

	SkilletTradeskillTooltip:AddLine("\n" .. self:GetReagentLabel(id));

	-- now the list of regents for this recipe and some info about them
	for i=1, 20, 1 do
		local reagent = s[i];
		if not reagent then
			break
		end

		local text = "  " .. reagent.needed .. " x " .. reagent.name;
		local reagent_counts = GRAY_FONT_COLOR_CODE .. " (" .. reagent.num .. " / " .. reagent.numwbank
		if reagent.numwalts then
			reagent_counts = reagent_counts .. " / " .. reagent.numwalts
		end
		reagent_counts = reagent_counts .. ")" .. FONT_COLOR_CODE_CLOSE
		if reagent.vendor then
			text = text .. GRAY_FONT_COLOR_CODE .. "  (" .. L["buyable"] .. ")" .. FONT_COLOR_CODE_CLOSE;
		end

		SkilletTradeskillTooltip:AddDoubleLine(text, reagent_counts, 1, 1, 1);
	end

	-- The legend at the bottom
    text =  "(" .. L["reagents in inventory"] .. " / " .. L["bank"]
    if s.numcraftablewithalts ~= nil then
        text = text .. " / " .. L["alts"]
    end
    text = text .. ")"
	SkilletTradeskillTooltip:AddDoubleLine("\n", text);

	SkilletTradeskillTooltip:Show();

end

-- Sets the game tooltip item to the selected skill
-- (and reagent at index if not nil)
function Skillet:SetTradeSkillToolTip(skill, index)
	GameTooltip:ClearLines();

	if Skillet:IsCraft() then
		if index then
			GameTooltip:SetCraftItem(skill, index)
		else
			GameTooltip:SetCraftSpell(skill)
		end
	else
		if index then
			GameTooltip:SetTradeSkillItem(skill, index)
		else
			GameTooltip:SetTradeSkillItem(skill)
		end
	end

	local s = self.stitch:GetItemDataByIndex(self.currentTrade, skill);

	-- Can the item be obtained from a vendor? Let the user know!
	if s and index and s[index].vendor then
		GameTooltip:AppendText(GRAY_FONT_COLOR_CODE .. " (" .. L["buyable"] .. ")" .. FONT_COLOR_CODE_CLOSE);
	end

end

-- Updates the details window with information about the currently selected skill
function Skillet:UpdateDetailsWindow(skill_index)
	if not skill_index or skill_index < 0 then
		SkilletSkillName:SetText("")
		SkilletSkillCooldown:SetText("")
		SkilletRequirementLabel:Hide()
		SkilletRequirementText:SetText("")
		SkilletSkillIcon:Hide()
		SkilletReagentLabel:Hide()
		SkilletRecipeNotesButton:Hide()
        SkilletPreviousItemButton:Hide()

		SkilletHighlightFrame:Hide()
		SkilletFrame.selectedSkill = -1;

		-- Always want these set.
		SkilletItemCountInputBox:SetText("1");
		SkilletCreateCountSlider:SetMinMaxValues(1, 20);
		SkilletCreateCountSlider:SetValue(1);

		for i=1, SKILLET_NUM_REAGENT_BUTTONS, 1 do
			local button = getglobal("SkilletReagent"..i);
			button:Hide();
		end
		return
	end

	SkilletFrame.selectedSkill = skill_index;
	self.numItemsToCraft = 1;

	if self.recipeNotesFrame then
		self.recipeNotesFrame:Hide()
	end

	local s = self.stitch:GetItemDataByIndex(self.currentTrade, skill_index);
--[[    
    local data = { GetTradeSkillItemStats(skill_index) }
    if data then
        self:Print(s.name)
        for k,v in pairs(data) do
            self:Print("  " .. v)
        end
        local id = self:GetItemIDFromLink(s.link)
        local itemName, itemLink, itemQuality, itemLevel, itemUseLevel, itemType, itemSubType, itemStackSize, itemEquipLoc, itemTexture = GetItemInfo(tonumber(id))
        self:Print("  (my) Requires level: " .. itemUseLevel)
    end
]]--    

	-- Name of the skill
	SkilletSkillName:SetText(s.name);
	SkilletRecipeNotesButton:Show();

	-- Whether or not it is in cooldown.
	if(GetTradeSkillCooldown(skill_index)) then
		SkilletSkillCooldown:SetText(COOLDOWN_REMAINING.." "..SecondsToTime(GetTradeSkillCooldown(skill_index)));
	else
		SkilletSkillCooldown:SetText("");
	end

	-- Are special tools needed for this skill?
	if s.tools then
		local text
		-- can't use s.tools here as GetCraftSpellFocus() needs an
		-- index rather than a name so it can do the lookups to see
		-- if we have the required item
		if Skillet:IsCraft() then
			text = BuildColoredListString(GetCraftSpellFocus(skill_index))
		else
			text = BuildColoredListString(GetTradeSkillTools(skill_index))
		end
		SkilletRequirementText:SetText(text)
		SkilletRequirementLabel:Show();
	else
		SkilletRequirementLabel:Hide();
		SkilletRequirementText:SetText("");
	end

	SkilletSkillIcon:SetNormalTexture(s.texture);
	SkilletSkillIcon:Show();

	-- How many can we queue/create?
	SkilletCreateCountSlider:SetMinMaxValues(1, max(20, s.numcraftablewbank));
	SkilletCreateCountSlider:SetValue(self.numItemsToCraft);
	SkilletItemCountInputBox:SetText("" .. self.numItemsToCraft);
	SkilletCreateCountSlider.tooltipText = L["Number of items to queue/create"];

	-- Reagents required ...
	SkilletReagentLabel:SetText(self:GetReagentLabel(SkilletFrame.selectedSkill));
	SkilletReagentLabel:Show();

	for i=1, SKILLET_NUM_REAGENT_BUTTONS, 1 do
		local       button = getglobal("SkilletReagent"..i);
		local  button_name = getglobal("SkilletReagent"..i.."Name");
		local button_count = getglobal("SkilletReagent"..i.."Count");

		local reagent = s[i];

		if reagent then
			local num = reagent.num;

			if ( num < reagent.needed ) then
				-- grey it out if we don't have it.
				SetItemButtonTextureVertexColor(button, 0.5, 0.5, 0.5);
				button_name:SetTextColor(GRAY_FONT_COLOR.r, GRAY_FONT_COLOR.g, GRAY_FONT_COLOR.b);
			else
				-- ungrey it
				SetItemButtonTextureVertexColor(button, 1.0, 1.0, 1.0);
				button_name:SetTextColor(HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b);
			end

			local text = reagent.name;

			if reagent.vendor then
				-- tooltip is also updated to append "buyable" to the end of the reagent name
				text = text .. GRAY_FONT_COLOR_CODE .. "*" .. FONT_COLOR_CODE_CLOSE
			end

			button_name:SetText(text);

			if ( num > 100 ) then
			 	-- don't have room to display more than 100 items.
				num = "*"
			end

			button_count:SetText(num .. "/" .. reagent.needed);
			SetItemButtonTexture(button, reagent.texture);

			button:Show();
			button_name:Show();
			button_count:Show();
		else
			-- out of necessary reagents, don't need to show the button,
			-- or any or the text.
			button:Hide();
			button_name:Hide();
			button_count:Hide();
		end

	end
    
    if self.db.profile.link_craftable_reagents and #previousRecipies > 0 then
        SkilletPreviousItemButton:Show()
    else
        SkilletPreviousItemButton:Hide()
    end

end

-- Updates the window/scroll list displaying queue of items
-- that are waiting to be crafted.
function Skillet:UpdateQueueWindow()
	local queue = self.stitch:GetQueueInfo();
	if not queue then
		return
	end
    
	local numItems = #queue;

	-- Update the scroll frame
	FauxScrollFrame_Update(SkilletQueueList,				-- frame
	                       numItems,                        -- num items
	                       SKILLET_QUEUE_ITEMS_DISPLAYED,   -- num to display
	                       SKILLET_TRADE_SKILL_HEIGHT)      -- value step (item height)
                           
	-- Where in the list of skill to start counting.
	local itemOffset = FauxScrollFrame_GetOffset(SkilletQueueList);

	-- Iterate through all the buttons that make up the scroll window
	-- and fill then in with data or hide them, as necessary
	for i=1, SKILLET_QUEUE_ITEMS_DISPLAYED, 1 do
		local itemIndex = i + itemOffset;

		local button       = getglobal("SkilletQueueButton"..i);
		local countFrame   = getglobal(button:GetName() .. "Count");
		local queueCount   = getglobal(button:GetName() .. "CountText");
		local nameFrame    = getglobal(button:GetName() .. "Name");
		local queueName    = getglobal(button:GetName() .. "NameText");
		local deleteButton = getglobal(button:GetName() .. "DeleteButton");

		-- Stick this on top of the button we use for displaying queue contents.
		deleteButton:SetFrameLevel(button:GetFrameLevel() + 1);

		local fixed_width = countFrame:GetWidth() + deleteButton:GetWidth()

		if ( SkilletQueueList:IsShown() ) then
			fixed_width = 275 - fixed_width - 10; -- 10 for the padding between items
		else
			fixed_width = 295 - fixed_width - 10; -- 10 for the padding between items
		end

		queueName:SetWidth(fixed_width);
		nameFrame:SetWidth(fixed_width);

		if itemIndex <= numItems then

			deleteButton:SetID(itemIndex);

            local s = self.stitch:GetQueueItemInfo(itemIndex)

            if s then
                queueName:SetText(s.name);
                queueCount:SetText(queue[itemIndex]["numcasts"]); --ick, can't find an API call for this.
            end

			queueName:Show();
			queueCount:Show();
			button:Show();
		else
			button:Hide();
			queueName:Hide();
			queueCount:Hide();
		end
	end
end

-- When one of the skill buttons in the left scroll pane is clicked
function Skillet:SkillButton_OnClick(button)
	if(button=="LeftButton") then
		local id = this:GetID();
		if id == -1 then
			-- header clicked / toggle collapsed state
			local header = this:GetText();
			local state = self.headerCollapsedState[header];

			if not state or state == false then
				self.headerCollapsedState[header] = true
			else
				self.headerCollapsedState[header] = false
			end
		else
			-- skill clicked
			self:SetSelectedSkill(id, true);

			-- if it was shift-left clicked *and* there is a chat edit
			-- window open, insert the recipe link.
			if IsShiftKeyDown() and (ChatFrameEditBox:IsVisible() or WIM_EditBoxInFocus ~= nil) then
				ChatEdit_InsertLink(self:GetTradeSkillRecipeLink(id));
			end
		end

		self:UpdateTradeSkillsWindow()
	end
end

-- Go to the previous recipe in the history list.
function Skillet:GoToPreviousRecipe()
    local itemID = table.remove(previousRecipies)
    if itemID then
        self:SetSelectedSkill(itemID);
    end
end

-- Called when then mouse enters a reagent button
function Skillet:ReagentButtonOnEnter(button, skill, index)
    if not self.db.profile.link_craftable_reagents then
        return
    end

    local s = self.stitch:GetItemDataByIndex(self.currentTrade, skill)
    local reagent = s[index];

    local can_craft = self.stitch:GetItemDataByName(reagent.name);

    if can_craft then
        gearTexture:SetParent(button)
        gearTexture:ClearAllPoints()
        gearTexture:SetPoint("TOPLEFT", button)
        gearTexture:Show()
    end
end

-- called then the mouse leaves a reagent button
function Skillet:ReagentButtonOnLeave(button, skill, index)
    gearTexture:Hide()
end

-- Called when the reagent button is clicked
function Skillet:ReagentButtonOnClick(button, skill, index)
    if not self.db.profile.link_craftable_reagents then
        return
    end
    
    -- current recipe -> selected reagent --> reagent item link
    local s = self.stitch:GetItemDataByIndex(self.currentTrade, skill)

    local reagent = self.stitch:GetItemDataByName(s[index].name);

    if reagent then
        -- we know how to make this, we just need to figure out
        -- what the *index* of the item is. That is not stored in
        -- the stitch library.

        for i=1, self:GetNumTradeSkills(), 1 do
            local item = self.stitch:GetItemDataByIndex(self.currentTrade, i)
            if item and item.link == reagent.link then
                table.insert(previousRecipies, skill)
                self:SetSelectedSkill(i)                
                return
            end
        end
    end

end

-- Updates the "Scanning tradeskill" text area with provided text
-- Set nil/empty text to hide the area
function Skillet:UpdateScanningText(text)
    local area = getglobal("SkilletFrameScanningText")
    if area then
        if text and string.len(text) > 0 then
            area:SetText(text)
            area:Show()
        else
            area:Hide()
        end
    end
end
